@@ -3,7 +3,7 @@
  */
 
 /*
- * Copyright (C) 2006-2019 Oracle Corporation
+ * Copyright (C) 2006-2020 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -61,7 +61,9 @@ typedef enum RTLOGGROUP
     RTLOGGROUP_DIR,
     RTLOGGROUP_FILE,
     RTLOGGROUP_FS,
+    RTLOGGROUP_FTP,
     RTLOGGROUP_HTTP,
+    RTLOGGROUP_IOQUEUE,
     RTLOGGROUP_LDR,
     RTLOGGROUP_LOCALIPC,
     RTLOGGROUP_PATH,
@@ -88,6 +90,9 @@ typedef enum RTLOGGROUP
  *
  *         If anyone might be wondering what the alphabet looks like:
  *              a b c d e f g h i j k l m n o p q r s t u v w x y z
+ *
+ * The RT_XX log group names are placeholders for new modules being added,
+ * to make sure that there always is a total of 32 log group entries.
  */
 #define RT_LOGGROUP_NAMES \
     "DEFAULT", \
@@ -97,7 +102,9 @@ typedef enum RTLOGGROUP
     "RT_DIR", \
     "RT_FILE", \
     "RT_FS", \
+    "RT_FTP", \
     "RT_HTTP", \
+    "RT_IOQUEUE", \
     "RT_LDR", \
     "RT_LOCALIPC", \
     "RT_PATH", \
@@ -108,8 +115,6 @@ typedef enum RTLOGGROUP
     "RT_TIME", \
     "RT_TIMER", \
     "RT_VFS", \
-    "RT_18", \
-    "RT_19", \
     "RT_20", \
     "RT_21", \
     "RT_22", \
@@ -121,7 +126,7 @@ typedef enum RTLOGGROUP
     "RT_28", \
     "RT_29", \
     "RT_30", \
-    "RT_ZIP"  \
+    "RT_ZIP"
 
 
 /** @def LOG_GROUP
@@ -147,7 +152,7 @@ typedef enum RTLOGGROUP
 #endif
 
 /** Logger structure. */
-#ifdef IN_RC
+#if defined(IN_RC) && !defined(DOXYGEN_RUNNING)
 typedef struct RTLOGGERRC RTLOGGER;
 #else
 typedef struct RTLOGGER RTLOGGER;
@@ -292,7 +297,7 @@ struct RTLOGGERRC
 
 
 
-#ifndef IN_RC
+#if !defined(IN_RC) || defined(DOXYGEN_RUNNING)
 
 /** Pointer to internal logger bits. */
 typedef struct RTLOGGERINTERNAL *PRTLOGGERINTERNAL;
@@ -335,7 +340,7 @@ struct RTLOGGER
 /** RTLOGGER::u32Magic value. (Avram Noam Chomsky) */
 # define RTLOGGER_MAGIC     UINT32_C(0x19281207)
 
-#endif /* !IN_RC */
+#endif /* !IN_RC || DOXYGEN_RUNNING */
 
 
 /**
@@ -489,7 +494,7 @@ RTDECL(void) RTLogPrintfEx(void *pvInstance, unsigned fFlags, unsigned iGroup,
 /** @def LOG_ENABLED
  * Use this compile time define to enable logging when not in debug mode
  * or LOG_DISABLED is set.
- * This will enabled Log() only.
+ * This will enable Log() only.
  */
 
 /** @def LOG_ENABLE_FLOW
@@ -614,7 +619,7 @@ RTDECL(void) RTLogPrintfEx(void *pvInstance, unsigned fFlags, unsigned iGroup,
 #  define _LogIt(a_fFlags, a_iGroup, ...) \
    do \
    { \
-        register PRTLOGGER LogIt_pLogger = RTLogDefaultInstanceEx(RT_MAKE_U32(a_fFlags, a_iGroup)); \
+        PRTLOGGER LogIt_pLogger = RTLogDefaultInstanceEx(RT_MAKE_U32(a_fFlags, a_iGroup)); \
         if (RT_LIKELY(!LogIt_pLogger)) \
         {   /* likely */ } \
         else \
@@ -628,7 +633,7 @@ RTDECL(void) RTLogPrintfEx(void *pvInstance, unsigned fFlags, unsigned iGroup,
 #  define LogIt(a_fFlags, a_iGroup, fmtargs) \
     do \
     { \
-        register PRTLOGGER LogIt_pLogger = RTLogDefaultInstanceEx(RT_MAKE_U32(a_fFlags, a_iGroup)); \
+        PRTLOGGER LogIt_pLogger = RTLogDefaultInstanceEx(RT_MAKE_U32(a_fFlags, a_iGroup)); \
         if (RT_LIKELY(!LogIt_pLogger)) \
         {   /* likely */ } \
         else \
@@ -639,7 +644,7 @@ RTDECL(void) RTLogPrintfEx(void *pvInstance, unsigned fFlags, unsigned iGroup,
 #  define LogItAlways(a_fFlags, a_iGroup, fmtargs) \
     do \
     { \
-        register PRTLOGGER LogIt_pLogger = RTLogDefaultInstanceEx(RT_MAKE_U32(0, UINT16_MAX)); \
+        PRTLOGGER LogIt_pLogger = RTLogDefaultInstanceEx(RT_MAKE_U32(0, UINT16_MAX)); \
         if (LogIt_pLogger) \
             LogIt_pLogger->pfnLogger fmtargs; \
     } while (0)
@@ -1225,7 +1230,7 @@ RTDECL(void) RTLogPrintfEx(void *pvInstance, unsigned fFlags, unsigned iGroup,
 
 
 
-/** @name Release Logging
+/** @defgroup grp_rt_log_rel    Release Logging
  * @{
  */
 
@@ -1702,7 +1707,7 @@ RTDECL(void) RTLogPrintfEx(void *pvInstance, unsigned fFlags, unsigned iGroup,
 /** @} */
 
 
-#ifndef IN_RC
+#if !defined(IN_RC) || defined(DOXYGEN_RUNNING)
 /**
  * Sets the default release logger instance.
  *
@@ -1710,7 +1715,7 @@ RTDECL(void) RTLogPrintfEx(void *pvInstance, unsigned fFlags, unsigned iGroup,
  * @param   pLogger     The new default release logger instance.
  */
 RTDECL(PRTLOGGER) RTLogRelSetDefaultInstance(PRTLOGGER pLogger);
-#endif /* !IN_RC */
+#endif
 
 /**
  * Gets the default release logger instance.
@@ -1798,7 +1803,7 @@ RTDECL(bool) RTLogRelSetBuffering(bool fBuffered);
 
 
 /** @name COM port logging
- * {
+ * @{
  */
 
 #ifdef DOXYGEN_RUNNING
@@ -1937,7 +1942,7 @@ RTDECL(PRTLOGGER)   RTLogGetDefaultInstance(void);
  */
 RTDECL(PRTLOGGER)   RTLogGetDefaultInstanceEx(uint32_t fFlagsAndGroup);
 
-#ifndef IN_RC
+#if !defined(IN_RC) || defined(DOXYGEN_RUNNING)
 /**
  * Sets the default logger instance.
  *
@@ -1945,7 +1950,7 @@ RTDECL(PRTLOGGER)   RTLogGetDefaultInstanceEx(uint32_t fFlagsAndGroup);
  * @param   pLogger     The new default logger instance.
  */
 RTDECL(PRTLOGGER)   RTLogSetDefaultInstance(PRTLOGGER pLogger);
-#endif /* !IN_RC */
+#endif
 
 #ifdef IN_RING0
 /**
@@ -1962,7 +1967,8 @@ RTDECL(int)         RTLogSetDefaultInstanceThread(PRTLOGGER pLogger, uintptr_t u
 #endif /* IN_RING0 */
 
 
-#ifndef IN_RC
+#if !defined(IN_RC) || defined(DOXYGEN_RUNNING)
+
 /**
  * Creates the default logger instance for a iprt users.
  *
@@ -2011,6 +2017,8 @@ RTDECL(int) RTLogCreate(PRTLOGGER *ppLogger, uint32_t fFlags, const char *pszGro
  * @param   cGroups             Number of groups in the array.
  * @param   papszGroups         Pointer to array of groups.  This must stick
  *                              around for the life of the logger instance.
+ * @param   cMaxEntriesPerGroup The max number of entries per group.  UINT32_MAX
+ *                              or zero for unlimited.
  * @param   fDestFlags          The destination flags.  RTLOGDEST_FILE is ORed
  *                              if pszFilenameFmt specified.
  * @param   pfnPhase            Callback function for starting logging and for
@@ -2028,11 +2036,11 @@ RTDECL(int) RTLogCreate(PRTLOGGER *ppLogger, uint32_t fFlags, const char *pszGro
  * @param   pszFilenameFmt      Log filename format string. Standard RTStrFormat().
  * @param   ...                 Format arguments.
  */
-RTDECL(int) RTLogCreateEx(PRTLOGGER *ppLogger, uint32_t fFlags, const char *pszGroupSettings,
-                          const char *pszEnvVarBase, unsigned cGroups, const char * const * papszGroups,
+RTDECL(int) RTLogCreateEx(PRTLOGGER *ppLogger, uint32_t fFlags, const char *pszGroupSettings, const char *pszEnvVarBase,
+                          unsigned cGroups, const char * const * papszGroups, uint32_t cMaxEntriesPerGroup,
                           uint32_t fDestFlags, PFNRTLOGPHASE pfnPhase, uint32_t cHistory,
                           uint64_t cbHistoryFileMax, uint32_t cSecsHistoryTimeSlot, PRTERRINFO pErrInfo,
-                          const char *pszFilenameFmt, ...) RT_IPRT_FORMAT_ATTR_MAYBE_NULL(13, 14);
+                          const char *pszFilenameFmt, ...) RT_IPRT_FORMAT_ATTR_MAYBE_NULL(14, 15);
 
 /**
  * Create a logger instance.
@@ -2048,6 +2056,8 @@ RTDECL(int) RTLogCreateEx(PRTLOGGER *ppLogger, uint32_t fFlags, const char *pszG
  * @param   cGroups             Number of groups in the array.
  * @param   papszGroups         Pointer to array of groups.  This must stick
  *                              around for the life of the logger instance.
+ * @param   cMaxEntriesPerGroup The max number of entries per group.  UINT32_MAX
+ *                              or zero for unlimited.
  * @param   fDestFlags          The destination flags.  RTLOGDEST_FILE is ORed
  *                              if pszFilenameFmt specified.
  * @param   pfnPhase            Callback function for starting logging and for
@@ -2066,11 +2076,11 @@ RTDECL(int) RTLogCreateEx(PRTLOGGER *ppLogger, uint32_t fFlags, const char *pszG
  *                              RTStrFormat().
  * @param   args                Format arguments.
  */
-RTDECL(int) RTLogCreateExV(PRTLOGGER *ppLogger, uint32_t fFlags, const char *pszGroupSettings,
-                           const char *pszEnvVarBase, unsigned cGroups, const char * const * papszGroups,
+RTDECL(int) RTLogCreateExV(PRTLOGGER *ppLogger, uint32_t fFlags, const char *pszGroupSettings, const char *pszEnvVarBase,
+                           unsigned cGroups, const char * const * papszGroups, uint32_t cMaxEntriesPerGroup,
                            uint32_t fDestFlags, PFNRTLOGPHASE pfnPhase, uint32_t cHistory,
                            uint64_t cbHistoryFileMax, uint32_t cSecsHistoryTimeSlot, PRTERRINFO pErrInfo,
-                           const char *pszFilenameFmt, va_list args) RT_IPRT_FORMAT_ATTR_MAYBE_NULL(13, 0);
+                           const char *pszFilenameFmt, va_list args) RT_IPRT_FORMAT_ATTR_MAYBE_NULL(14, 0);
 
 /**
  * Create a logger instance for singled threaded ring-0 usage.
@@ -2215,7 +2225,8 @@ RTDECL(int) RTLogGetGroupSettings(PRTLOGGER pLogger, char *pszBuf, size_t cchBuf
  * @param   pszValue    Value to parse.
  */
 RTDECL(int) RTLogGroupSettings(PRTLOGGER pLogger, const char *pszValue);
-#endif /* !IN_RC */
+
+#endif /* !IN_RC || DOXYGEN_RUNNING */
 
 /**
  * Updates the flags for the logger instance using the specified
@@ -2254,7 +2265,8 @@ RTDECL(bool) RTLogSetBuffering(PRTLOGGER pLogger, bool fBuffered);
  */
 RTDECL(uint32_t) RTLogSetGroupLimit(PRTLOGGER pLogger, uint32_t cMaxEntriesPerGroup);
 
-#ifndef IN_RC
+#if !defined(IN_RC) || defined(DOXYGEN_RUNNING)
+
 /**
  * Get the current log flags as a string.
  *
@@ -2280,7 +2292,6 @@ RTDECL(int) RTLogDestinations(PRTLOGGER pLogger, char const *pszValue);
  *
  * @returns IPRT status code.
  * @param   pLogger             Logger instance (NULL for default logger).
- * @param   pszValue            The value to parse.
  * @param   pErrInfo            Where to return extended error info.  Optional.
  */
 RTDECL(int) RTLogClearFileDelayFlag(PRTLOGGER pLogger, PRTERRINFO pErrInfo);
@@ -2295,7 +2306,8 @@ RTDECL(int) RTLogClearFileDelayFlag(PRTLOGGER pLogger, PRTERRINFO pErrInfo);
  *                              than 0.
  */
 RTDECL(int) RTLogGetDestinations(PRTLOGGER pLogger, char *pszBuf, size_t cchBuf);
-#endif /* !IN_RC */
+
+#endif /* !IN_RC || DOXYGEN_RUNNING */
 
 /**
  * Flushes the specified logger.
